from django import VERSION
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from six import string_types

from .components import Dropdown
from .views import ModelToolsView


def patterns(prefix, *args):
    if VERSION < (1, 9):
        from django.conf.urls import patterns as django_patterns
        return django_patterns(prefix, *args)
    elif prefix != '':
        raise Exception("You need to update your URLConf to be a list of URL "
                        "objects")
    else:
        return list(args)


class AdminRowActionsMixin(object):
    
    """ModelAdmin mixin to add row actions just like adding admin actions"""

    rowactions = []
    _named_row_actions = {}

    @property
    def media(self):
        media = super(AdminRowActionsMixin, self).media
        media.add_js(['js/jquery.dropdown.min.js'])
        media.add_css({'all': ['css/jquery.dropdown.min.css']})
        return media
        
    def get_list_display(self, request):
        list_display = super(AdminRowActionsMixin, self).get_list_display(request)
        if '_row_actions' not in list_display:
            list_display += ('_row_actions',)
        return list_display
    
    def get_actions_list(self, obj, includePk=True):
        
        def to_dict(tool_name):
            return dict(
                name=tool_name,
                label=getattr(tool, 'label', tool_name).replace('_', ' ').title(),
            )
        
        items = []
        
        row_actions = self.get_row_actions(obj)
        url_prefix = '{}/'.format(obj.pk if includePk else '')
        
        for tool in row_actions:
            if isinstance(tool, string_types):  # Just a str naming a callable
                tool_dict = to_dict(tool)
                items.append({
                    'label': tool_dict['label'],
                    'url': '{}rowactions/{}/'.format(url_prefix, tool),
                })
                
            elif isinstance(tool, dict):  # A parameter dict
                tool['enabled'] = tool.get('enabled', True)
                if 'action' in tool:  # If 'action' is specified then use our generic url in preference to 'url' value
                    if isinstance(tool['action'], tuple):
                        self._named_row_actions[tool['action'][0]] = tool['action'][1]
                        tool['url'] = '{}rowactions/{}/'.format(url_prefix, tool['action'][0])
                    else:
                        tool['url'] = '{}rowactions/{}/'.format(url_prefix, tool['action'])
                items.append(tool)
        
        return items
    
    def _row_actions(self, obj):
        
        items = self.get_actions_list(obj)
        if items:
            html = Dropdown(
                label=_("Actions"),
                items=items,
            ).render()

            return html
        return ''
    _row_actions.short_description = ''
    _row_actions.allow_tags = True
    
    def get_tool_urls(self):
        
        """Gets the url patterns that route each tool to a special view"""
        
        my_urls = patterns(
            '',
            url(r'^(?P<pk>\d+)/rowactions/(?P<tool>\w+)/$',
                self.admin_site.admin_view(ModelToolsView.as_view(model=self.model))
            )
        )
        return my_urls
    
    ###################################
    # EXISTING ADMIN METHODS MODIFIED #
    ###################################
    
    def get_urls(self):
        
        """Prepends `get_urls` with our own patterns"""
        
        urls = super(AdminRowActionsMixin, self).get_urls()
        return self.get_tool_urls() + urls
    
    ##################
    # CUSTOM METHODS #
    ##################
    
    def get_row_actions(self, obj):
        return getattr(self, 'rowactions', False) or []
    
    def get_change_actions(self, request, object_id, form_url):
    
        # If we're also using django_object_actions
        # then try to reuse row actions as object actions
    
        change_actions = super(AdminRowActionsMixin, self).get_change_actions(request, object_id, form_url)
        
        # Make this reuse opt-in
        if getattr(self, 'reuse_row_actions_as_object_actions', False):

            obj = self.model.objects.get(pk=object_id)
            row_actions = self.get_actions_list(obj, False) if obj else []
    
            for row_action in row_actions:
                # Object actions only supports strings as action indentifiers
                if isinstance(row_action, string_types):
                    change_actions.append(row_action)
                elif isinstance(row_action, dict):
                    if isinstance(row_action['action'], string_types):
                        change_actions.append(row_action['action'])
                    elif isinstance(row_action['action'], tuple):
                        change_actions.append(str(row_action['action'][1]))
        return change_actions



from django.contrib import admin
from django.contrib import messages
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.views.generic import View
from django.views.generic.detail import SingleObjectMixin


class ModelToolsView(SingleObjectMixin, View):
    
    """A special view that run the tool's callable"""

    def get(self, request, **kwargs):
        
        # SingleObjectMixin's `get_object`. Works because the view
        # is instantiated with `model` and the urlpattern has `pk`.
        
        obj = self.get_object()
        model_admin = admin.site._registry[obj.__class__]
        
        # Look up the action in the following order:
        # 1. in the named_row_actions dict (for lambdas etc)
        # 2. as a method on the model admin
        # 3. as a method on the model
        if kwargs['tool'] in model_admin._named_row_actions:
            action_method = model_admin._named_row_actions[kwargs['tool']]
            ret = action_method(request=request, obj=obj)
        elif getattr(model_admin, kwargs['tool'], False):
            action_method = getattr(model_admin, kwargs['tool'])
            ret = action_method(request=request, obj=obj)  # TODO should the signature actually be (obj, request) for consistancy?
        elif getattr(obj, kwargs['tool'], False):
            action_method = getattr(obj, kwargs['tool'])
            ret = action_method()
        else:
            raise Http404
        
        # If the method returns a response use that,
        # otherwise redirect back to the url we were called from
        if isinstance(ret, HttpResponse):
            response = ret
        else:
            back = request.META['HTTP_REFERER']
            response = HttpResponseRedirect(back)
            
        return response

    # Also allow POST
    post = get

    def message_user(self, request, message):
        # Copied from django.contrib.admin.options
        # Included to mimic admin actions
        messages.info(request, message)



from setuptools import find_packages
from setuptools import setup

setup(
    name='django-admin-row-actions',
    version='0.0.2',
    description='django admin row actions',
    author='Andy Baker',
    author_email='andy@andybak.net',
    url='https://github.com/DjangoAdminHackers/django-admin-row-actions',
    packages=find_packages(),
    install_requires=[
        'six',
    ],
    package_data={
        'django_admin_row_actions': [
            'static/css/*.css',
            'static/js/*.js',
            'templates/django_admin_row_actions/*.html',
        ]
    },
    include_package_data=True,
)



import weakref
from django.template.loader import render_to_string


class BaseComponent(object):
    
    template = None
    instances = []

    def __init__(self, **kwargs):
        
        # Add ourselves to the class's list of instances
        # so we can generate a unique id number for the html
        
        self.__class__.instances.append(weakref.proxy(self))
        self.context = kwargs
        self.context['dom_id'] = self.get_unique_id()

    @classmethod
    def get_unique_id(cls):
        return "{}-{}".format(cls.__name__.lower(), len(cls.instances))

    def render(self):
        return render_to_string(
            self.template,
            self.context,
        )

    def __unicode__(self):
        return self.render()


class Dropdown(BaseComponent):

    template = 'django_admin_row_actions/dropdown.html'

    def __init__(self, **kwargs):
        super(Dropdown, self).__init__(**kwargs)



from functools import wraps
from django.db.models.query import QuerySet


class QuerySetIsh(QuerySet):
    
    """Takes an instance and mimics it coming from a QuerySet"""
    
    def __init__(self, instance=None, *args, **kwargs):
        try:
            model = instance._meta.model
        except AttributeError:
            # Django 1.5 does this instead, getting the model may be overkill
            # we may be able to throw away all this logic
            model = instance._meta.concrete_model
        self._doa_instance = instance
        super(QuerySetIsh, self).__init__(model, *args, **kwargs)
        self._result_cache = [instance]

    def _clone(self, *args, **kwargs):
        # don't clone me, bro
        return self

    def get(self, *args, **kwargs):
        # Starting in Django 1.7, `QuerySet.get` started slicing to `MAX_GET_RESULTS`,
        # so to avoid messing with `__getslice__`, override `.get`.
        return self._doa_instance


def takes_instance_or_queryset(func):
    
    """Decorator that makes standard actions compatible"""
    
    @wraps(func)
    def decorated_function(self, request, queryset):
        # Function follows the prototype documented at:
        # https://docs.djangoproject.com/en/dev/ref/contrib/admin/actions/#writing-action-functions
        if not isinstance(queryset, QuerySet):
            queryset = QuerySetIsh(queryset)
        return func(self, request, queryset)
    return decorated_function
